#include <zephyr.h>
#include <logging/log.h>
#include <net/openthread.h>
#include <net/socket.h>
#include <openthread/thread.h>
#include <stdio.h>

#include <mgmt/mcumgr/smp_udp.h>
#include <net/net_mgmt.h>
#include <net/net_event.h>
#include <net/net_conn_mgr.h>

#include "os_mgmt/os_mgmt.h"

LOG_MODULE_REGISTER(main, CONFIG_MAIN_LOG_LEVEL);

#define EVENT_MASK (NET_EVENT_L4_CONNECTED | NET_EVENT_L4_DISCONNECTED)

static struct net_mgmt_event_callback mgmt_cb;

static void event_handler(struct net_mgmt_event_callback *cb,
			  uint32_t mgmt_event, struct net_if *iface)
{
	if ((mgmt_event & EVENT_MASK) != mgmt_event) {
		return;
	}

	if (mgmt_event == NET_EVENT_L4_CONNECTED) {
		LOG_INF("Network connected");

		if (smp_udp_open() < 0) {
			LOG_ERR("could not open smp udp");
		}

		return;
	}

	if (mgmt_event == NET_EVENT_L4_DISCONNECTED) {
		LOG_INF("Network disconnected");
		smp_udp_close();
		return;
	}
}


void get_ieee_eui64(uint8_t *ieee_eui64)
{
	uint64_t factory_address;
	uint32_t index = 0;
	// By default, the device uses Nordic Semiconductor's MA-L (fa-ce-36). It can be modified by overwriting
	const uint32_t nordic_oui = 0xf4ce36;

	// Set the MAC Address Block Larger (MA-L) formerly called OUI.
	ieee_eui64[index++] = (nordic_oui >> 16) & 0xff;
	ieee_eui64[index++] = (nordic_oui >> 8) & 0xff;
	ieee_eui64[index++] = nordic_oui & 0xff;

	// Use device identifier assigned during the production.
	factory_address = (uint64_t)NRF_FICR->DEVICEID[0] << 32;
	factory_address |= NRF_FICR->DEVICEID[1];
	memcpy(ieee_eui64 + index, &factory_address, sizeof(factory_address) - index);
}

void get_macaddress(uint8_t *macaddr)
{
	uint8_t eui64[8];
	get_ieee_eui64(eui64);
	// get the last 6 bytes, ignoring the first two bytes of Nordic Semiconductor's MA-L
	memcpy(macaddr, &eui64[2], 6);
}

void insert_ip_addr(){
	// add manual ip, CUSTOM PREFIX + macaddress
	uint8_t macaddr[6];
	char manual_ip[46];
	
	get_macaddress(macaddr);

	sprintf(manual_ip, "fdaa:bb:1::%02x%02x:%02x%02x:%02x%02x",
		macaddr[0], macaddr[1], macaddr[2], macaddr[3],
		macaddr[4], macaddr[5]);
	
	LOG_INF("Adding manual ip: %s", manual_ip);

	otNetifAddress aAddress;

	otIp6AddressFromString(manual_ip, &aAddress.mAddress);
	aAddress.mPrefixLength  = 64;
	aAddress.mPreferred     = true;
	aAddress.mValid         = true;
	aAddress.mAddressOrigin = OT_ADDRESS_ORIGIN_MANUAL;
	otIp6AddUnicastAddress(openthread_get_default_context()->instance, &aAddress);
}

void main(void)
{
	insert_ip_addr();

	// init SMP
	os_mgmt_register_group();

	// init SMP UDP
	LOG_INF("STARTING SMP UDP");
	net_mgmt_init_event_callback(&mgmt_cb, event_handler, EVENT_MASK);
	net_mgmt_add_event_callback(&mgmt_cb);
	net_conn_mgr_resend_status();

	LOG_INF("SMP: build time: " __DATE__ " " __TIME__);

	LOG_INF("Device fully initialized!");

}
